@@ -93,12 +93,13 @@ import tempfile
 import platform
 import subprocess as sp
 
+from calendar import monthrange
 from time import sleep
 from datetime import time, date, datetime, timedelta
 from collections import OrderedDict
 
 __pkgname__ = 'python-crontab'
-__version__ = '2.7.1'
+__version__ = '3.0.0'
 
 ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)'
                      r'\s+([^@#\s]+)\s+([^\n]*?)(\s+#\s*([^\n]*)|$)')
@@ -158,22 +159,6 @@ if not WINOS:
         """Returns the username of the current user"""
         return pwd.getpwuid(os.getuid())[0]
 
-def open_pipe(cmd, *args, **flags):
-    """Runs a program and orders the arguments for compatability.
-
-    a. keyword args are flags and always appear /before/ arguments for bsd
-    """
-    cmd_args = tuple(shlex.split(cmd, posix=flags.pop('posix', POSIX)))
-    env = flags.pop('env', None)
-    for (key, value) in flags.items():
-        if len(key) == 1:
-            cmd_args += (("-%s" % key),)
-            if value is not None:
-                cmd_args += (str(value),)
-        else:
-            cmd_args += (("--%s=%s" % (key, value)),)
-    args = tuple(arg for arg in (cmd_args + tuple(args)) if arg)
-    return sp.Popen(args, stdout=sp.PIPE, stderr=sp.PIPE, env=env)
 
 def _str(text):
     """Convert to the best string format for this python version"""
@@ -182,6 +167,53 @@ def _str(text):
     return text
 
 
+class Process:
+    """Runs a program and orders the arguments for compatability.
+
+    a. keyword args are flags and always appear /before/ arguments for bsd
+    """
+    def __init__(self, cmd, *args, **flags):
+        cmd_args = tuple(shlex.split(cmd, posix=flags.pop('posix', POSIX)))
+        self.env = flags.pop('env', None)
+        for (key, value) in flags.items():
+            if len(key) == 1:
+                cmd_args += (f"-{key}",)
+                if value is not None:
+                    cmd_args += (str(value),)
+            else:
+                cmd_args += (f"--{key}={value}",)
+        self.args = tuple(arg for arg in (cmd_args + tuple(args)) if arg)
+        self.has_run = False
+        self.stdout = None
+        self.stderr = None
+        self.returncode = None
+
+    def _run(self):
+        """Run this process and return the popen process object"""
+        return sp.Popen(self.args, stdout=sp.PIPE, stderr=sp.PIPE, env=self.env)
+
+    def run(self):
+        """Run this process and store whatever is returned"""
+        process = self._run()
+        (out, err) = process.communicate()
+        self.returncode = process.returncode
+        self.stdout = out.decode("utf-8")
+        self.stderr = err.decode("utf-8")
+        return self
+
+    def __str__(self):
+        return self.stdout.strip()
+
+    def __repr__(self):
+        return f"Process({self.args})"
+
+    def __int__(self):
+        return self.returncode
+
+    def __eq__(self, other):
+        return str(self) == other
+
+
 class CronTab:
     """
     Crontab object which can access any time based cron using the standard.
@@ -252,7 +284,7 @@ class CronTab:
         elif name == 'crons' and value:
             raise AttributeError("You can NOT set crons attribute directly")
         else:
-            super(CronTab, self).__setattr__(name, value)
+            super().__setattr__(name, value)
 
     def read(self, filename=None):
         """
@@ -273,12 +305,10 @@ class CronTab:
                 lines = fhl.readlines()
 
         elif self.user:
-            (out, err) = open_pipe(self.cron_command, l='', **self.user_opt).communicate()
-            if err and 'no crontab for' in str(err):
-                pass
-            elif err:
-                raise IOError("Read crontab %s: %s" % (self.user, err))
-            lines = out.decode('utf-8').split("\n")
+            process = Process(self.cron_command, l='', **self.user_opt).run()
+            if process.stderr and 'no crontab for' not in process.stderr:
+                raise IOError(f"Read crontab {self.user}: {process.stderr}")
+            lines = process.stdout.split("\n")
 
         self.lines = lines
 
@@ -305,8 +335,8 @@ class CronTab:
                 cron_id = self.crons.index(before)
                 line_id = self.lines.index(before)
 
-        except ValueError:
-            raise ValueError("Can not find CronItem in crontab to insert before")
+        except ValueError as err:
+            raise ValueError("Can not find CronItem in crontab to insert before") from err
 
         if item.is_valid():
             item.env.update(self._parked_env)
@@ -349,7 +379,7 @@ class CronTab:
                 return
 
         if self.filen:
-            fileh = open(self.filen, 'wb')
+            fileh = open(self.filen, 'wb') # pylint: disable=consider-using-with
         else:
             filed, path = tempfile.mkstemp()
             fileh = os.fdopen(filed, 'wb')
@@ -363,11 +393,11 @@ class CronTab:
                 os.unlink(path)
                 raise IOError("Please specify user or filename to write.")
 
-            proc = open_pipe(self.cron_command, path, **self.user_opt)
+            proc = Process(self.cron_command, path, **self.user_opt)._run()
             ret = proc.wait()
             if ret != 0:
-                raise IOError("Program Error: {} returned {}: {}".format(
-                    self.cron_command, ret, proc.stderr.read()))
+                msg = proc.stderr.read()
+                raise IOError(f"Program Error: {self.cron_command} returned {ret}: {msg}")
             proc.stdout.close()
             proc.stderr.close()
             os.unlink(path)
@@ -383,17 +413,17 @@ class CronTab:
             if ret not in [None, -1]:
                 yield ret
 
-    def run_scheduler(self, timeout=-1, **kwargs):
+    def run_scheduler(self, timeout=-1, cadence=60, warp=False):
         """Run the CronTab as an internal scheduler (generator)"""
         count = 0
         while count != timeout:
             now = datetime.now()
-            if 'warp' in kwargs:
+            if warp:
                 now += timedelta(seconds=count * 60)
             for value in self.run_pending(now=now):
                 yield value
 
-            sleep(kwargs.get('cadence', 60))
+            sleep(cadence)
             count += 1
 
     def render(self, errors=False, specials=True):
@@ -413,7 +443,7 @@ class CronTab:
                 elif not errors:
                     crons.append('# DISABLED LINE\n# ' + line)
                 else:
-                    raise ValueError("Invalid line: %s" % line)
+                    raise ValueError(f"Invalid line: {line}")
             elif isinstance(line, CronItem):
                 if not line.is_valid() and not errors:
                     line.enabled = False
@@ -421,12 +451,12 @@ class CronTab:
 
         # Environment variables are attached to cron lines so order will
         # always work no matter how you add lines in the middle of the stack.
-        result = str(self.env) + u'\n'.join(crons)
-        if result and result[-1] not in (u'\n', u'\r'):
-            result += u'\n'
+        result = str(self.env) + '\n'.join(crons)
+        if result and result[-1] not in ('\n', '\r'):
+            result += '\n'
         return result
 
-    def new(self, command='', comment='', user=None, pre_comment=False, before=None):
+    def new(self, command='', comment='', user=None, pre_comment=False, before=None): # pylint: disable=too-many-arguments
         """
         Create a new CronItem and append it to the cron.
 
@@ -541,12 +571,12 @@ class CronTab:
     def __repr__(self):
         kind = 'System ' if self._user is False else ''
         if self.filen:
-            return "<%sCronTab '%s'>" % (kind, self.filen)
+            return f"<{kind}CronTab '{self.filen}'>"
         if self.user and not self.user_opt:
             return "<My CronTab>"
         if self.user:
-            return "<User CronTab '%s'>" % self.user
-        return "<Unattached %sCronTab>" % kind
+            return f"<User CronTab '{self.user}'>"
+        return f"<Unattached {kind}CronTab>"
 
     def __iter__(self):
         """Return generator so we can track jobs after removal"""
@@ -695,23 +725,24 @@ class CronItem:
         if not self.is_valid() and self.enabled:
             raise ValueError('Refusing to render invalid crontab.'
                              ' Disable to continue.')
-        command = _str(self.command).replace(u'%', u'\\%')
+        command = _str(self.command).replace('%', '\\%')
         user = ''
         if self.cron and self.cron.user is False:
             if not self.user:
                 raise ValueError("Job to system-cron format, no user set!")
             user = self.user + ' '
-        result = u"%s %s%s" % (self.slices.render(specials=specials), user, command)
+        rend = self.slices.render(specials=specials)
+        result = f"{rend} {user}{command}"
         if self.stdin:
             result += ' %' + self.stdin.replace('\n', '%')
         if not self.enabled:
-            result = u"# " + result
+            result = "# " + result
         if self.comment:
             comment = self.comment = _str(self.comment)
             if self.marker:
-                comment = u"#%s: %s" % (self.marker, comment)
+                comment = f"#{self.marker}: {comment}"
             else:
-                comment = u"# " + comment
+                comment = "# " + comment
 
             if SYSTEMV or self.pre_comment or self.stdin:
                 result = comment + "\n" + result
@@ -757,6 +788,34 @@ class CronItem:
         """
         return self.slices.frequency(year=year)
 
+    def frequency_at_hour(self, year=None, month=None, day=None, hour=None):
+        """Returns the number of times this item will execute in a given hour
+           (defaults to this hour)
+        """
+        return self.slices.frequency_at_hour(year=year, month=month, day=day, hour=hour)
+
+    def frequency_at_day(self, year=None, month=None, day=None):
+        """Returns the number of times this item will execute in a given day
+           (defaults to today)
+        """
+        return self.slices.frequency_at_day(year=year, month=month, day=day)
+
+    def frequency_at_month(self, year=None, month=None):
+        """Returns the number of times this item will execute in a given month
+           (defaults to this month)
+        """
+        return self.slices.frequency_at_month(year=year, month=month)
+
+    def frequency_at_year(self, year=None):
+        """Returns the number of times this item will execute in a given year
+           (defaults to this year)
+        """
+        return self.slices.frequency_at_year(year=year)
+
+    def frequency(self, year=None):
+        """Return frequence per year times frequency per day"""
+        return self.frequency_per_year(year=year) * self.frequency_per_day()
+
     def frequency_per_year(self, year=None):
         """Returns the number of /days/ this item will execute on in a year
            (defaults to this year)
@@ -789,10 +848,10 @@ class CronItem:
         env = os.environ.copy()
         env.update(self.env.all())
         shell = self.env.get('SHELL', SHELL)
-        (out, err) = open_pipe(shell, '-c', self.command, env=env).communicate()
-        if err:
-            LOG.error(err.decode("utf-8"))
-        return out.decode("utf-8").strip()
+        process = Process(shell, '-c', self.command, env=env).run()
+        if process.stderr:
+            LOG.error(process.stderr)
+        return process
 
     def schedule(self, date_from=None):
         """Return a croniter schedule if available."""
@@ -801,9 +860,9 @@ class CronItem:
         try:
             # Croniter is an optional import
             from croniter.croniter import croniter # pylint: disable=import-outside-toplevel
-        except ImportError:
+        except ImportError as err:
             raise ImportError("Croniter not available. Please install croniter"
-                              " python module via pip or your package manager")
+                              " python module via pip or your package manager") from err
         return croniter(self.slices.clean_render(), date_from, ret_type=datetime)
 
     def description(self, **kw):
@@ -814,9 +873,9 @@ class CronItem:
         """
         try:
             from cron_descriptor import ExpressionDescriptor # pylint: disable=import-outside-toplevel
-        except ImportError:
+        except ImportError as err:
             raise ImportError("cron_descriptor not available. Please install"\
-              "cron_descriptor python module via pip or your package manager")
+              "cron_descriptor python module via pip or your package manager") from err
 
         exdesc = ExpressionDescriptor(self.slices.clean_render(), **kw)
         return exdesc.get_description()
@@ -874,7 +933,7 @@ class CronItem:
         return self.slices[4]
 
     def __repr__(self):
-        return "<CronItem '%s'>" % str(self)
+        return f"<CronItem '{self}'>"
 
     def __len__(self):
         return len(str(self))
@@ -922,7 +981,7 @@ class Every:
     def year(self):
         """Special every year target"""
         if self.unit > 1:
-            raise ValueError("Invalid value '%s', outside 1 year" % self.unit)
+            raise ValueError(f"Invalid value '{self.unit}', outside 1 year")
         self.slices.setall('@yearly')
 
 
@@ -932,7 +991,7 @@ class CronSlices(list):
         month requency and finally day of the week frequency.
      """
     def __init__(self, *args):
-        super(CronSlices, self).__init__([CronSlice(info) for info in S_INFO])
+        super().__init__([CronSlice(info) for info in S_INFO])
         self.special = None
         self.setall(*args)
         self.is_valid = self.is_self_valid
@@ -977,7 +1036,8 @@ class CronSlices(list):
             # It might be possible to later understand timedelta objects
             # but there's no convincing mathematics to do the conversion yet.
         if not isinstance(value, (list, tuple)):
-            raise ValueError("Unknown type: {}".format(type(value).__name__))
+            typ = type(value).__name__
+            raise ValueError(f"Unknown type: {typ}")
         return value, None
 
     @staticmethod
@@ -986,10 +1046,10 @@ class CronSlices(list):
         key = value.lstrip('@').lower()
         if value.count(' ') == 4:
             return value.strip().split(' '), None
-        if key in SPECIALS.keys():
+        if key in SPECIALS:
             return SPECIALS[key].split(' '), '@' + key
         if value.startswith('@'):
-            raise ValueError("Unknown special '{}'".format(value))
+            raise ValueError(f"Unknown special '{value}'")
         return [value], None
 
     def clean_render(self):
@@ -1006,7 +1066,7 @@ class CronSlices(list):
         if not SYSTEMV and specials is True:
             for (name, value) in SPECIALS.items():
                 if value == slices and name not in SPECIAL_IGNORE:
-                    return "@%s" % name
+                    return f"@{name}"
         return slices
 
     def clear(self):
@@ -1045,6 +1105,82 @@ class CronSlices(list):
         """Returns the number of times this item will execute in any hour"""
         return len(self[0])
 
+    def frequency_at_year(self, year=None):
+        """Returns the number of /days/ this item will execute
+           in a given year (default is this year)"""
+        if not year:
+            year = date.today().year
+
+        total = 0
+        for month in range(1, 13):
+            total += self.frequency_at_month(year, month)
+        return total
+
+    def frequency_at_month(self, year=None, month=None):
+        """Returns the number of times this item will execute in given month
+        (default: current month)
+        """
+        if year is None and month is None:
+            year = date.today().year
+            month = date.today().month
+        elif year is None or month is None:
+            raise ValueError(
+                f"One of more arguments undefined: year={year}, month={month}")
+
+        total = 0
+        if month in self[3]:
+            # Calculate amount of days of specific month
+            days = monthrange(year, month)[1]
+            for day in range(1, days + 1):
+                total += self.frequency_at_day(year, month, day)
+        return total
+
+    def frequency_at_day(self, year=None, month=None, day=None):
+        """Returns the number of times this item will execute in a day
+        (default: any executed day)
+        """
+        # If arguments provided, all needs to be provided
+        test_none = [x is None for x in [year, month, day]]
+
+        if all(test_none):
+            return len(self[0]) * len(self[1])
+
+        if any(test_none):
+            raise ValueError(
+                f"One of more arguments undefined: year={year}, month={month}, day={day}")
+
+        total = 0
+        if day in self[2]:
+            for hour in range(24):
+                total += self.frequency_at_hour(year, month, day, hour)
+        return total
+
+    def frequency_at_hour(self, year=None, month=None, day=None, hour=None):
+        """Returns the number of times this item will execute in a hour
+        (default: any executed hour)
+        """
+        # If arguments provided, all needs to be provided
+        test_none = [x is None for x in [year, month, day, hour]]
+
+        if all(test_none):
+            return len(self[0])
+
+        if any(test_none):
+            raise ValueError(
+                f"One of more arguments undefined: year={year}, month={month}, day={day}, hour={hour}")
+
+        result = 0
+        weekday = date(year, month, day).weekday()
+
+        # Check if scheduled for execution at defined moment
+        if hour in self[1] and \
+           day in self[2] and \
+           month in self[3] and \
+           ((weekday + 1) % 7) in self[4]:
+            result = len(self[0])
+
+        return result
+
     def __str__(self):
         return self.render()
 
@@ -1098,7 +1234,7 @@ class CronSlice:
                     continue
                 self.parts.append(self.parse_value(part, sunday=0))
 
-    def render(self, resolve=False, specials=True):
+    def render(self, resolve=False):
         """Return the slice rendered as a crontab.
 
         resolve - return integer values instead of enums (default False)
@@ -1109,7 +1245,7 @@ class CronSlice:
         return _render_values(self.parts, ',', resolve)
 
     def __repr__(self):
-        return "<CronSlice '%s'>" % str(self)
+        return f"<CronSlice '{self}'>"
 
     def __eq__(self, value):
         return str(self) == str(value)
@@ -1182,10 +1318,10 @@ class CronSlice:
             val = self.min
         try:
             out = get_cronvalue(val, self.enum)
-        except ValueError:
-            raise ValueError("Unrecognised %s: '%s'" % (self.name, val))
-        except KeyError:
-            raise KeyError("No enumeration for %s: '%s'" % (self.name, val))
+        except ValueError as err:
+            raise ValueError(f"Unrecognised {self.name}: '{val}'") from err
+        except KeyError as err:
+            raise KeyError(f"No enumeration for {self.name}: '{val}'") from err
 
         if self.max == 6 and int(out) == 7:
             if sunday is not None:
@@ -1193,7 +1329,7 @@ class CronSlice:
             raise SundayError("Detected Sunday as 7 instead of 0!")
 
         if int(out) < self.min or int(out) > self.max:
-            raise ValueError("'{1}', not in {0.min}-{0.max} for {0.name}".format(self, val))
+            raise ValueError(f"'{val}', not in {self.min}-{self.max} for {self.name}")
         return out
 
 
@@ -1240,7 +1376,7 @@ def _render(value, resolve=False):
         return value.render(resolve)
     if resolve:
         return str(int(value))
-    return str(u'{:02d}'.format(value) if ZERO_PAD else value)
+    return str(f'{value:02d}' if ZERO_PAD else value)
 
 
 class CronRange:
@@ -1286,11 +1422,11 @@ class CronRange:
                     self.dangling = 0
                 self.vto = self.slice.parse_value(vto, sunday=6)
             if self.vto < self.vfrom:
-                raise ValueError("Bad range '{0.vfrom}-{0.vto}'".format(self))
+                raise ValueError(f"Bad range '{self.vfrom}-{self.vto}'")
         elif value == '*':
             self.all()
         else:
-            raise ValueError('Unknown cron range value "%s"' % value)
+            raise ValueError(f'Unknown cron range value "{value}"')
 
     def all(self):
         """Set this slice to all units between the miniumum and maximum"""
@@ -1306,7 +1442,7 @@ class CronRange:
             else:
                 value = _render_values([self.vfrom, self.vto], '-', resolve)
         if self.seq != 1:
-            value += "/%d" % self.seq
+            value += f"/{self.seq:d}"
         if value != '*' and SYSTEMV:
             value = ','.join([str(val) for val in self.range()])
         return value
@@ -1344,7 +1480,7 @@ class OrderedVariableList(OrderedDict):
     """
     def __init__(self, *args, **kw):
         self.job = kw.pop('job', None)
-        super(OrderedVariableList, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
 
     @property
     def previous(self):
@@ -1370,10 +1506,10 @@ class OrderedVariableList(OrderedDict):
     def __getitem__(self, key):
         previous = self.previous
         if key in self:
-            return super(OrderedVariableList, self).__getitem__(key)
+            return super().__getitem__(key)
         if previous is not None:
             return previous.all()[key]
-        raise KeyError("Environment Variable '%s' not found." % key)
+        raise KeyError(f"Environment Variable '{key}' not found.")
 
     def __str__(self):
         """Constructs to variable list output used in cron jobs"""
@@ -1383,7 +1519,7 @@ class OrderedVariableList(OrderedDict):
                 if self.previous.all().get(key, None) == value:
                     continue
             if ' ' in str(value) or value == '':
-                value = '"%s"' % value
-            ret.append("%s=%s" % (key, str(value)))
+                value = f'"{value}"'
+            ret.append(f"{key}={value}")
         ret.append('')
         return "\n".join(ret)
