@@ -116,9 +116,9 @@ class CompatTestCase(unittest.TestCase):
 
     def test_07_non_posix_shell(self):
         """Shell in windows environments is split correctly"""
-        from crontab import open_pipe
+        from crontab import Process
         winfile = os.path.join(TEST_DIR, 'data', "bash\\win.exe")
-        pipe = open_pipe("{sys.executable} {winfile}".format(winfile=winfile, sys=sys), 'SLASHED', posix=False)
+        pipe = Process("{sys.executable} {winfile}".format(winfile=winfile, sys=sys), 'SLASHED', posix=False)._run()
         self.assertEqual(pipe.wait(), 0, 'Windows shell command not found!')
         (out, err) = pipe.communicate()
         self.assertEqual(out, b'Double Glazing Installed:SLASHED\n')
