@@ -62,7 +62,7 @@ crontabs.KNOWN_LOCATIONS = [
     (crontabs.AnaCronTab, os.path.join(TEST_DIR, 'data', 'anacron')),
 ]
 
-crontab.CRONCMD = "%s %s" % (sys.executable, os.path.join(TEST_DIR, 'data', 'crontest'))
+crontab.CRON_COMMAND = "%s %s" % (sys.executable, os.path.join(TEST_DIR, 'data', 'crontest'))
 
 class CronTabsTestCase(unittest.TestCase):
     """Test use documentation in crontab."""
