@@ -25,15 +25,12 @@ import sys
 sys.path.insert(0, '../')
 
 import unittest
-from crontab import CronTab, PY3
+from crontab import CronTab
 try:
     from test import test_support
 except ImportError:
     from test import support as test_support
 
-if PY3:
-    unicode = str
-
 START_TAB = """
 3 * * * * command1 # CommentID C
 2 * * * * command2 # CommentID AAB
@@ -57,7 +54,7 @@ class RemovalTestCase(unittest.TestCase):
         """Remove All"""
         self.crontab.remove_all()
         self.assertEqual(len(self.crontab), 0)
-        self.assertEqual(unicode(self.crontab), '')
+        self.assertEqual(str(self.crontab), '')
 
     def test_03_remove_cmd(self):
         """Remove all with Command"""
