@@ -34,6 +34,7 @@ except ImportError:
 TEST_FILE = os.path.join(os.path.dirname(__file__), 'data', 'crontab')
 INITIAL_TAB = """
 */30 * * * * palin one_cross_each
+*/30 * * * * matias\ttab_user_error
 """
 
 class SystemCronTestCase(unittest.TestCase):
@@ -47,21 +48,20 @@ class SystemCronTestCase(unittest.TestCase):
 
     def test_01_read(self):
         """Read existing command"""
-        jobs = 0
-        for job in self.crontab:
-            self.assertEqual(job.user, 'palin')
-            self.assertEqual(job.command, 'one_cross_each')
-            jobs += 1
-        self.assertEqual(jobs, 1)
+        job = self.crontab[0]
+        self.assertEqual(job.user, 'palin')
+        self.assertEqual(job.command, 'one_cross_each')
+        job = self.crontab[1]
+        self.assertEqual(job.user, 'matias')
+        self.assertEqual(job.command, 'tab_user_error')
+        self.assertEqual(len(self.crontab), 2)
 
     def test_02_new(self):
         """Create a new job"""
         job = self.crontab.new(command='release_brian', user='pontus')
         self.assertEqual(job.user, 'pontus')
         self.assertEqual(job.command, 'release_brian')
-        self.assertEqual(str(self.crontab), """
-*/30 * * * * palin one_cross_each
-
+        self.assertEqual(str(self.crontab), INITIAL_TAB.replace('\t', ' ') + """
 * * * * * pontus release_brian
 """)
 
@@ -88,6 +88,7 @@ class SystemCronTestCase(unittest.TestCase):
         self.crontab._user = None
         self.assertEqual(str(self.crontab), """
 */30 * * * * one_cross_each
+*/30 * * * * tab_user_error
 """)
         self.crontab.new(command='now_go_away')
 
@@ -110,7 +111,7 @@ class SystemCronTestCase(unittest.TestCase):
         for i in range(10):
             self.crontab = CronTab(tab=str(self.crontab))
 
-        self.assertEqual(INITIAL_TAB, str(self.crontab))
+        self.assertEqual(INITIAL_TAB.replace('\t', ' '), str(self.crontab))
 
     def test_10_system_file(self):
         """Load system crontab from a file"""
