@@ -210,13 +210,14 @@ class UseTestCase(unittest.TestCase):
         self.assertEqual(cronitem.render(specials=None), '@daily true')
         self.assertEqual(cronitem.render(specials=False), '0 0 * * * true')
 
-    def test_25_open_pipe(self):
+    def test_25_process(self):
         """Test opening pipes"""
-        from crontab import open_pipe, CRON_COMMAND
-        pipe = open_pipe(CRON_COMMAND, h=None, a='one', abc='two')
-        (out, err) = pipe.communicate()
-        self.assertEqual(err, b'')
-        self.assertEqual(out, b'--abc=two|-a|-h|one\n')
+        from crontab import Process, CRON_COMMAND
+        process = Process(CRON_COMMAND, h=None, a='one', abc='two').run()
+        self.assertEqual(int(process), 0)
+        self.assertEqual(repr(process)[:8], "Process(")
+        self.assertEqual(process.stderr, '')
+        self.assertEqual(process.stdout, '--abc=two|-a|-h|one\n')
 
     def test_07_zero_padding(self):
         """Can we get zero padded output"""
